/* ==== ajoin.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h"
#include "mg_utils.h"
#include "crash.h"
#include "dXrng.h"
#include "ui.h"
#include "ajoin.h" /* Needs to be put elsewhere */

extern roomdata_t room[];
/* makes more sense than using a function */
#define CLASSIFY_RPOS(sx1,sy1,sx2,sy2,tx1,ty1,tx2,ty2) ( (sy2-1<ty1+1 || ty2-1<sy1+1)? ( (sx2-1<tx1+1 || tx2-1<sx1+1)? 0:1) :2)

/* When going from a tunnel to a room */
#define CLASSIFY_TRPOS(sx,sy, tx1,ty1, tx2,ty2) ( (1*(sx>tx1&&sx<tx2))+ (2*(sy>ty1&&sy<ty2))+ (3*(sx==tx1||sx==tx2))+ (4*(sy==ty1||sy==ty2)))
/* if sx==tx1 or sx==tx2, give 3.
   if sy==ty1 or sy==ty2, give 4.
   if (sx>sx1 && sx<sx2)  give 1
   if (sy>sx1 && sy<sy2)  give 2. Else give 0. */
/* The converse (from a room to a tunnel) needs no special treatment:
   the previous sx,sy (from the room source) needs no changing, and
   the point found with a tunnel is an acceptable target.*/

/* We will have to be able to deal the fact that tunnels can join 2 areas
  by merely running *adjacent* to another tunnel.
  More disturbingly, can join MORE than 2 areas at once!!!
  Potentially 4 areas.
  Does it actually *MATTER* if we join more than 2? It merely makes it
  a bit more connected. The map is not going to become 100% tunnel
  just by this, and we wont have the problem of unaccessible areas.*/


void attempt_joining(int source,int target)
{
    /* starts trying to connect source to target.
       If it hits something else on the way, then will
       either try to connect source to it, or it to target.
       If a tunnel, connect by coords. If a room, connect
       by room value.*/
    int sx,sy; /* Points to try to send tunnel from*/
    int tx,ty; /* Points to try to send tunnel to*/
    int mx,my; /* Points to try to place corner at*/
    int px,py,p; /* temp vars used in probing */
    int stype=2; /* 0 if source is tunnel, 1 if source is room, 2 if new room*/
    int ttype=2; /* 0 if target is tunnel, 1 if target is room, 2 if new room*/
    int sx1,sx2,sy1,sy2; /* Corners of source room*/
    int tx1,tx2,ty1,ty2; /* Corners of target room*/
    int xstart,xend,ystart,yend; /*constraints for x and y values of tunnels*/
    direction_t dir; /* a temporary variable used in probing.*/
    direction_t dir1,dir2; /* Direction of tunnels used for type 0 paths*/
    int pathstate=0; /* 1 indicates we've scanned sx,sy->mx,my, and have
      begun scanning mx,my->tx,ty */
    area_t *source_area=room[source].Area; /*source_area never changes*/
    area_t *target_area=room[target].Area; /*target_area may change*/
    area_t *probe_area;
    static dXrng_roller *d20=NULL;
    int relat;
    int strike=0; /*When we've found a clear path from somewhere
        to somewhere else, strike=1*/

    if(target==-1) {show_messages("Target=-1!");crash(OUT_OF_BOUNDS);}
    if(source==-1) {show_messages("Source=-1!");crash(OUT_OF_BOUNDS);}
    if (NULL==d20)d20=new_dX(20);
    if (room[source].Xcentre==-1){
      sprintf(strbuf,"Room %d doesnt exist!",source);
      show_messages(strbuf);
      crash(INCONSISTENT);
    }
    if (room[target].Xcentre==-1){
      sprintf(strbuf,"Room %d doesnt exist!",target);
      show_messages(strbuf);
      crash(INCONSISTENT);
    }
    while(0==strike)
    {
        if(2==stype)
        {
           sx1=room[source].Xstart; sx2=sx1+(room[source].Width)-1;
           sy1=room[source].Ystart; sy2=sy1+(room[source].Height)-1;
        }
        if(2==ttype)
        {
           tx1=room[target].Xstart; tx2=tx1+(room[target].Width)-1;
           ty1=room[target].Ystart; ty2=ty1+(room[target].Height)-1;
        }

        if(3<=(stype+ttype))
        { /* Plan course from new room to some other room */
            relat=CLASSIFY_RPOS(sx1,sy1,sx2,sy2, tx1,ty1, tx2,ty2);
            switch (relat) {
               case 0:
                 /* We need a corner(mx,my) and 2 tunnels to it*/
                 if(roll_dX(d20)<10)
                 {/* Move horizontal first, then vertical*/
                    dir1=(sx1<tx1)?FACE_E:FACE_W;
                    dir2=(sy1<ty1)?FACE_S:FACE_N;
                    sx=(FACE_W==dir1)?sx1:sx2;
                    ty=(FACE_N==dir2)?ty2:ty1;
                    xstart=tx1+1;xend=tx2-1;
                    ystart=sy1+1;yend=sy2-1;
                    sy=ystart+((yend-ystart)*roll_dX(d20)/19);
                    tx=xstart+((xend-xstart)*roll_dX(d20)/19);
                    mx=tx;   my=sy;
                    pathstate=0;
                 } else { /* Move vertical first, then horizontal*/
                    dir1=(sy1<ty1)?FACE_S:FACE_N;
                    dir2=(sx1<tx1)?FACE_E:FACE_W;
                    sy=(FACE_N==dir1)?sy1:sy2;
                    tx=(FACE_W==dir2)?tx2:tx1;
                    xstart=sx1+1;xend=sx2-1;
                    ystart=ty1+1;yend=ty2-1;
                    sx=xstart+((xend-xstart)*roll_dX(d20)/19);
                    ty=ystart+((yend-ystart)*roll_dX(d20)/19);
                    mx=sx;  my=ty;
                    pathstate=0;
                 }
                 break;
               case 1:
                 /* source is n or s of target*/
                 xstart=MAX(sx1,tx1)+1;xend=MIN(sx2,tx2)-1;
                 sx=xstart+((xend-xstart)*roll_dX(d20)/19);/*Somewhere between*/
                 /* We ought to make sure that sx,sy is not *next to*
                 an existing door. Can be ON one tho.
                 Ditto tx,ty! Is this possible? OHO! If tx,ty is a door,
                 there will be a tunnel already coming in the opposite
                 direction, and our new tunnel will meet this and connect
                 to it BEFORE we reach our target!!!
                 Furthermore, I realise now, that if sx,sy is next to
                 a door, the tunnel from that will run next to where
                 our new tunnel wants to, so we will actually start the
                 tunnel where the previous one ended!*/
                 tx=sx;
                 dir=(sy1<ty1)?FACE_S:FACE_N;
                 sy=(FACE_N==dir)?sy1:sy2;
                 ty=(FACE_N==dir)?ty2:ty1;
                 break;
               case 2:
                 /* source is w or e of target*/
                 ystart=MAX(sy1,ty1)+1;yend=MIN(sy2,ty2)-1;
                 sy=ystart+((yend-ystart)*roll_dX(d20)/19);
                 ty=sy;
                 dir=(sx1<tx1)?FACE_E:FACE_W;
                 sx=(FACE_W==dir)?sx1:sx2;
                 tx=(FACE_W==dir)?tx2:tx1;
                 break;
            }
        }
        if (ttype==2 &&!stype)
        { /* Working out a path from part of a tunnel to a new room */
            relat=CLASSIFY_TRPOS(sx,sy, tx1,ty1,tx2,ty2);
            if (!relat) relat=(roll_dX(d20)<10)?3:4;
            switch (relat) {
               case 1:
                 /* source n or s of target */
                 dir=(sy<ty1)?FACE_S:FACE_N;
                 ty=(FACE_N==dir)?ty2:ty1;
                 tx=sx; /*Oh. That's it??*/
                 break;
               case 2:
                 /* source w or e of target */
                 dir=(sx<tx1)?FACE_E:FACE_W;
                 tx=(FACE_W==dir)?tx2:tx1;
                 ty=sy;
                 break;
               case 3:
                 /* Must go H before V to target */
                 /* Pick a point on target wall */
                 xstart=tx1+1;xend=tx2-1;
                 tx=xstart+((xend-xstart)*roll_dX(d20)/19);
                 dir1=(sx<tx)?FACE_E:FACE_W;
                 dir2=(sy<ty1)?FACE_S:FACE_N;
                 ty=(FACE_N==dir2)?ty2:ty1;
                 mx=tx;   my=sy;
                 pathstate=0;
                 break;
               case 4:
                 /* Must go V before H to target */
                 /* Pick a point on target wall */
                 ystart=ty1+1;yend=ty2-1;
                 ty=ystart+((yend-ystart)*roll_dX(d20)/19);
                 dir1=(sy<ty)?FACE_S:FACE_N;
                 dir2=(sx<tx1)?FACE_E:FACE_W;
                 tx=(FACE_W==dir2)?tx2:tx1;
                 mx=sx;  my=ty;
                 pathstate=0;
                 break;
            }

        }
        if (stype) stype=1;
        if (ttype) ttype=1;
        /* Now we know *where* we want to look, we look! */
        switch (relat){
           case 3:
           case 4:
           case 0: /* For probing+building, 0,3,4 are equivalent.*/
             /* We need a corner (mx,my), and 2 tunnels to it*/
             /* probe direction 1 first. If probe goes past mx,my,
             then change to phase 2.
             If phase 2, probe direction 2. If probe goes past tx,ty,
             strike!*/
             dir=(pathstate)?dir2:dir1;
             px=(pathstate)?mx:sx;
             py=(pathstate)?my:sy;
             p=probe_path(px,py,dir);
             px=(pathstate)?tx:mx;
             py=(pathstate)?ty:my;
             if (target_reached(p,px,py,dir,!pathstate))
             { /* reached whichever our target was */
                if (pathstate) strike=1; /* We have a total path!*/
                else pathstate=1; /* Check the next half of path */
             } else {/* hit an obstacle.*/
                /* If it was already reachable, update stype,
                then "source" if a room, or "sx,sy" if a tunnel.
                pathstate, relat, etc SHOULD update themselves
                next iteration.
                If NOT yet reachable, we try to connect to it. By, er...*/
                if(dir==FACE_N||dir==FACE_S) /* As case 1*/
                { if(probe_status(px,p,dir)==0)
                  { p=find_room(px,p);
                    if (p==-1) {show_messages("find_room=-1!");crash(NO_MEM);}
                    if(room[p].Area==source_area)
                    { source=p;stype=2; /* We don't change pathstate,etc*/
                    } else { /* as they will get altered by new room*/
                      target=p;ttype=2; /* being discovered*/
                    }
                  } else { /* We found tunnels */
                    probe_area=get_area(px,p,source_area);
                    if (probe_area==source_area)
                    { sy=p;stype=0; /* Source is tunnel */
                      if (pathstate) {pathstate=0;sx=px;relat=1;}
                    } else {
                       ty=p;ttype=0; /* Ah. Now what sort of path is it?
                         For tunnels, we *don't* make a further pass!*/
                       if (pathstate)
                        strike=1; else{relat=1;tx=sx;strike=1;}
                    }
                  }
                } else { /* Facing w or e. As case 2*/
                  if(probe_status(p,py,dir)==0)
                  { p=find_room(p,py);
                    if(p==-1){show_messages("find_room=-1!");crash(NO_MEM);}
                    if(room[p].Area==source_area)
                    { source=p;stype=2;
                    } else {
                      target=p;ttype=2;
                    }
                  } else { /* found tunnels */
                    probe_area=get_area(p,py,source_area);
                    if (probe_area==source_area)
                    {  sx=p;stype=0;
                       if(pathstate) {pathstate=0;sy=py;relat=2;}
                    } else {
                       tx=p; ttype=0;
                       if (pathstate)
                        strike=1;else {relat=1;ty=sy;strike=1;}
                    }
                  }
                }
             }
             break;
           case 1: /* source is n or s of target */
             p=probe_path(sx,sy,dir);
             if(target_reached(p,tx,ty,dir,0))
             {  strike=1; /* yay!*/
             } else{
                if(probe_status(sx,p,dir)==0)
                {  p=find_room(sx,p);
                   if(p==-1){show_messages("find_room=-1!");crash(NO_MEM);}
                   if(room[p].Area==source_area)
                   {  source=p;
                      stype=2;/* New source room*/
                   } else {
                      target=p;
                      ttype=2;/* Next iteration will get the proper tx,ty*/
                   }
                } else { /* We found tunnels around */
                   probe_area=get_area(sx,p,source_area);
                   if (probe_area==source_area)
                   {  sy=p;stype=0; /* Source is tunnel */
                   } else {
                      ty=p;ttype=0;strike=1;
                   }
                }
             }
             break;
           case 2: /* source is w or e of target */
             /* Probe dir . If probe goes past tx,ty, strike!*/
             p=probe_path(sx,sy,dir);
             if(target_reached(p,tx,ty,dir,0))
             {  strike=1;
             } else {
                if(probe_status(p,sy,dir)==0)
                {  p=find_room(p,sy);
                   if(p==-1){show_messages("find_room=-1!");crash(NO_MEM);}
                   if(room[p].Area==source_area)
                   {  source=p;
                      stype=2;/* New source room*/
                   } else {
                      target=p;
                      ttype=2;
                   }
                } else { /* found tunnels */
                   probe_area=get_area(p,sy,source_area);
                   if(probe_area==source_area)
                   {   sx=p;stype=0;
                   } else {
                       tx=p;ttype=0;strike=1;
                   }
                }
             }
             break;
        }

    }
    /* we have identified source *and* target, each of which may be
      either a room, or a point in a tunnel*/
    if (ttype)target_area=room[target].Area;
    else target_area=get_area(tx,ty,source_area);

    if(relat>0&&relat<3) /*3 and 4 are now as 0!!*/
    { /* 1 straight path */
         build_tunnel(source_area,sx,sy,tx,ty);

    } else { /* 2 tunnels joined with a corner at (mx,my)*/
         build_tunnel(source_area,sx,sy,mx,my);
         build_tunnel(source_area,mx,my,tx,ty);
    }
    if(stype) {build_door(sx,sy);} /* build a door at sx,sy*/
    if(ttype) {build_door(tx,ty);} /* build a door at tx,ty*/
    /* If we leave a tunnel tile where a door would be, it'll confuse
      functions that classify squares. But will it matter? Will
      anything find that square? Oh yeah, it could. In which case, lets
      have the option of placing a room tile there? */
    merge_areas(source_area,target_area);
    /* If we have a type 0 relation (needs a corner), then source
      sends a tunnel to somewhere (mx,my) parallel with target.
      If it hits something already connected to source, tries to 
      send tunnel from that something to (mx,my) and continue.
      If it hits something not yet connected to source, send
      tunnel to that something and return.
      If it hits nothing, then try searching from (mx,my) to
      target. If it hits something already connected to source,
      then DONT send anything from (sx,sy) to (mx,my), but try
      to send tunnel from (something) to target.
      So, once we find something between (mx,my) and (tx,ty),
      if it is in source, we change relat to 1 or 2 accordingly
      and set sx=mx and sy=my. DOH! No, set sx,sy to obstacle.
      */
    /* End of function!*/
}

int target_reached(int probe,int targetx,int targety,direction_t dir, int mp)
{
    /* If probe has got to or past targetx,targety,
    if midpoint, returns 1 if probe is PAST target.
    if not midpoint, returns 1 if probe has REACHED target. */
    if (targetx<2||targetx>=MAPWID-2||targety<2||targety>=MAPHGT-2)
    {
      sprintf(strbuf,"Targetting bad coords?");
      show_messages(strbuf);
      crash(OUT_OF_BOUNDS);
    }
    switch (dir) {
       case FACE_N:
         return ((probe<=(targety-mp))?1:0);
         break;
       case FACE_E:
         return ((probe>=(targetx+mp))?1:0);
         break;
       case FACE_S:
         return ((probe>=(targety+mp))?1:0);
         break;
       case FACE_W:
         return ((probe<=(targetx-mp))?1:0);
         break;
    }
    show_messages("Somehow reached the end of target_reached() function.");
    crash(FUNCTION_ERROR);
    return(0); /* Shut up, gcc... */
}

int probe_status(int x,int y, direction_t dir)
{
   /* When a probe returns, you don't know what it found to make it return.
   pass the returned coords (+ direction) to this to find out if it was:
   -A tunnel directly below the cursor (means nothing adjacent can be
   connected to a different area),
   -A room type square (will really mean either a perimeter wall, a door,
     or a corner, unless I've made a foobar)
   -Next to exactly one tunnel square
   -Next to more tunnels .
   What then?
   If a tunnel, would find its area ptr.
   If a room, would find its number
   If next to a tunnel, would find its area ptr.
   If next to more tunnels, would JOIN THEM first, then find the NEW area
   ptr for them. Well, if choosing to tunnel to/from there! If not, then...
   Aha! Could call a function to see if current area is one of those.
   Such a function could be used also for the other cases!*/
   /* 0=room. 1=tunnel. 2=adjacent tunnel. 3=adjacent tunnelS */
   maploc_t *mapstart=map->Data;
   tile_t ttile;
   int count=0;
   ttile=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
   if (ttile==PTL_CORNER||ttile==PTL_DOOR||ttile==PTL_VWALL||ttile==PTL_HWALL) return(0);
   if (ttile==PTL_FLOOR) return (0); /*This shoud NOT happen! But seems to*/
   if (ttile==PTL_TUNNEL) return(1);
   switch(dir) {
      case FACE_N:
      case FACE_S:
         ttile=(mapstart+FOOGRID_IDX(map,y,x-1))->Tiletype;
         if(ttile==PTL_TUNNEL) count=1;
         ttile=(mapstart+FOOGRID_IDX(map,y,x+1))->Tiletype;
         if(ttile==PTL_TUNNEL) count++;
         return(count);
      case FACE_W:
      case FACE_E:
         ttile=(mapstart+FOOGRID_IDX(map,y-1,x))->Tiletype;
         if(ttile==PTL_TUNNEL) count=1;
         ttile=(mapstart+FOOGRID_IDX(map,y+1,x))->Tiletype;
         if(ttile==PTL_TUNNEL) count++;
         return(count);
   }
   show_messages("Somehow reached the end of probe_status() function.");
   crash(FUNCTION_ERROR);
   return(0); /* Shut up again, gcc! */
}

/* if either (x,y) or at least one of its orthogonal neighbours has a
   different area to this, return such area. Otherwise, return this
   if any of them has this area, or NULL if no areas found. */
/* Why? Because if we find a point bordered by an already connected
   area and a not-yet-connected area, then we would be better off
   building a tunnel TO it, not from it. Trying to build from it
   would not automatically link the areas (as far as the algorithm's
   concerned)*/
area_t *get_area(int x,int y,area_t *this)
{
   /* Search for other. */
/*   tile_t n,s,w,e;  The tiles around the centre. Which aren't checked.*/
   tile_t centre;
   area_t *na;
   area_t *sa;
   area_t *wa;
   area_t *ea;
   maploc_t *mapstart=map->Data;
   if (y<3||x<3||x>=MAPWID-2||y>=MAPHGT-2)
   {
      show_messages("Get_area for dubious coords!");
      crash(OUT_OF_BOUNDS);
   }
   centre=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
   if (centre!=PTL_ROCK)
   {
   /* We've found *something* on the target square. Anything
     else around must be connected to it.*/
      if (centre==PTL_TUNNEL) return(find_area(x,y)); /* Works for tunnels*/
      else return (room[find_room(x,y)].Area); /* Works for rooms!*/
   }
   /* NOW we're sure that there could be something adjacent that's
      not yet connected. find_area now doesn't consider rooms, thank
      goodness. */
   na=(find_area(x,y-1));
   sa=(find_area(x,y+1));
   wa=(find_area(x-1,y));
   ea=(find_area(x+1,y));
   if(NULL!=na&&this!=na) return(na);
   if(NULL!=sa&&this!=sa) return(sa);
   if(NULL!=wa&&this!=wa) return(wa);
   if(NULL!=ea&&this!=ea) return(ea);
   /* No "other" areas found.*/
   if(NULL!=na) return(na);
   if(NULL!=sa) return(sa);
   if(NULL!=wa) return(wa);
   if(NULL!=ea) return(ea);
   crash(INCONSISTENT);
   return(NULL);/*SURELY there MUST have been a more concise way to say that?*/
}

void build_door(int x,int y)
{
   maploc_t *mapstart=map->Data;
   (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_DOOR;
}
