/* ==== ctrl.c ==== */
/* runs the main loop: getting keypresses, doing actions
   corresponding to the keypresses, updating other things,
   then displaying the map */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "porrog_defs.h"
#include <ncurses.h>
#include <unistd.h>
#include <stddef.h>
#include "C-foogrid.h" /* Used for foogrid*/

#include "mapgen_types.h"
#include "maparray.h"
#include "crash.h"
#include "dXrng.h"
#include "memutils.h"

#include "item_t.h"
#include "structures.h"

#include "item_p.h"
#include "player.h"
#include "monster.h"
#include "version.h"
#include "ui.h"
#include "apply.h"


/* Returns different values depending on why the game exited,
   for main() to deal with appropriately */
int ctrl_loop() /* Should this take any args? */
{
    /* Initial coordinates of the top-left corner of the display */
    int x=(MAPWID/2)-25,y=(MAPHGT/2)-10,quit=0;
    char k;

/*  make_map();  Should get called when you enter a new level */

    while(quit==0)
    {
      draw_map(x,y,player.X,player.Y);
        /* Somewhere in the middle */
      k=getch();
      switch (k){
         case '?': /* Help! */
            show_messages("This should give help.");
            break;
         case 'h':
            p_move(map,-1,0); /* Now p_move();*/
            if (player.X<(x+16)) {x=(player.X-26);}
            if(x<0) x=0;
            break;
         case 'y': /* -1,-1 */
            p_move(map,-1,-1);
            if (player.X<(x+16)) {x=(player.X-26);}
            if (player.Y<(y+7))  {y=(player.Y-13);}
            if(x<0) x=0;
            if(y<0) y=0;
            break;
         case 'u': /* 1,-1 */
            p_move(map,1,-1);
            if (player.X>(x+32)) {x=(player.X-24);}
            if (player.Y<(y+7))  {y=(player.Y-13);}
            if(x>(MAPWID-51)) x=(MAPWID-50);
            if(y<0) y=0;
            break;
         case 'j':
            p_move(map,0,1);
            if (player.Y>(y+15)) {y=(player.Y-10);}
            if(y>(MAPHGT-23)) y=(MAPHGT-23);
            break;
         case 'k':
            p_move(map,0,-1);
            if (player.Y<(y+7)) {y=(player.Y-13);}
            if(y<0) y=0;
            break;
         case 'b':
            p_move(map,-1,1);
            if (player.X<(x+16)) {x=(player.X-26);}
            if (player.Y>(y+15)) {y=(player.Y-10);}
            if(x<0) x=0;
            if(y>(MAPHGT-23)) y=(MAPHGT-23);
            break;
         case 'n':
            p_move(map,1,1);
            if (player.X>(x+32)) {x=(player.X-24);}
            if (player.Y>(y+15)) {y=(player.Y-10);}
            if(x>(MAPWID-51)) x=(MAPWID-50);
            if(y>(MAPHGT-23)) y=(MAPHGT-23);
            break;
         case 'l':
            p_move(map,1,0);
            if (player.X>(x+32)) {x=(player.X-24);}
            if(x>(MAPWID-51)) x=(MAPWID-50);
            break;
         case 'o':
            /* open doors*/
            p_open_door(map); /* now p_open_door()*/
            break;
         case 'a':
            /* Apply something! New code. */
            apply_main();
            break;
         case 'q': /* Should set return status */
            quit=1;
            break;
         case 'r': /* Should soon change this to do reading */
            sprintf(strbuf,"Room %d",find_room(player.X,player.Y));
            show_messages(strbuf);
            break;
         case 'v':
            version(strbuf);
            show_messages(strbuf);
            break;
         case 'i':
            display_heap("Your inventory",player.Inventlist);
            break;
         case ';':
            display_heap("On the floor",((maploc_t *)(map->Data)+FOOGRID_IDX(map,player.Y,player.X))->Itemlist);
            break;
         case ',': /* Will now choose an item to pick up */
            p_pickup(map); /* Does it all now */
            break;
         case 'd': /* Will now choose an item to drop */
            p_drop(map);
            break;
         case 'w': /* Wield summat */
            show_messages("'w'ield doesn't work yet!");
            break;
         case 'W': /* Wear summat */
            show_messages("'W'ear doesn't work yet!");
            break;
         case 'M':
            show_messages("This is a very very long message designed to test the show_messages() function, you see?");
            break;
         case 'm':
            show_messages("Creating new map.");
            make_map();
            break;
         default:
      }
      /* Now update environment */
    }
    /* We've exited. Either quitted, saved, died, or won. Other options? */
    show_messages("Quitting..?");
/*  endwin(); */
/*  printf("%ld unfreed allocations made.\n",check_allocs()); */
    return(0); /* Change to whatever our return status is */
}
