/* ==== i_init.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */




/* This initialises the set of item types. Still not sure if such
   data should be inside the .c source or a header... No, it
   would make no sense to put it in a header, as it would only
   be #included once. */

/* Need helper functions that do more than just "new_object_type",
 and define type-specific things */

#include "porrog_defs.h"
#include "item_t.h"
#include "structures.h"
#include "C-foogrid.h" /* dammit we HAVE to separate protos from typedefs. */
#include "item_p.h"

#define DEF_WEAP(name) push_itype(IG_WEAPON,name);
#define DEF_BOW(name) push_itype(IG_RANGED,name);
#define DEF_MISSILE(name) push_itype(IG_MISSILE,name);
#define DEF_SUIT(name) push_itype(IG_SUIT,name);
#define DEF_SHIELD(name) push_itype(IG_SHIELD,name);
#define DEF_HEADGEAR(name) push_itype(IG_HEADGEAR,name);
#define DEF_GLOVES(name) push_itype(IG_GLOVES,name);
#define DEF_FOOTWEAR(name) push_itype(IG_FOOTWEAR,name);
#define DEF_RING(name) push_itype(IG_RING,name);
#define DEF_NECKWEAR(name) push_itype(IG_NECKWEAR,name);
#define DEF_SCROLL(name)  push_itype(IG_SCROLL,name);
#define DEF_BOOK(name) push_itype(IG_BOOK,name);
#define DEF_FOOD(name) push_itype(IG_FOOD,name);
#define DEF_CONDIMENT(name) push_itype(IG_CONDIMENT,name);
#define DEF_POTION(name)  push_itype(IG_POTION,name);
#define DEF_WAND(name)  push_itype(IG_WAND,name);
#define DEF_ROD(name)   push_itype(IG_ROD,name);
#define DEF_TOOL(name)  push_itype(IG_TOOL,name);
#define DEF_CONTAINER(name)  push_itype(IG_CONTAINER,name);
#define DEF_STONE(name)  push_itype(IG_STONE,name);
#define DEF_COINAGE(name)  push_itype(IG_COINAGE,name);

void i_init()
{
    /* Wieldables/wearables */
    DEF_WEAP("sword");
    DEF_WEAP("axe");
    DEF_WEAP("spear");
    DEF_WEAP("club");
    DEF_WEAP("knife");
    DEF_WEAP("claw hammer");
    DEF_WEAP("mallet");
    DEF_WEAP("sledgehammer");
    DEF_WEAP("exacto knife");
    DEF_WEAP("penknife");
    DEF_WEAP("hatchet");
    DEF_WEAP("cleaver");

    DEF_BOW("bow");
    DEF_BOW("crossbow");

    DEF_MISSILE("arrow");
    DEF_MISSILE("bolt");

    DEF_SUIT("pinstripe suit");
    DEF_SUIT("boiler suit");
    DEF_SUIT("gimp suit");
    DEF_SUIT("leather armour");
    DEF_SUIT("chain mail");
    DEF_SUIT("plate mail");
    DEF_SUIT("kevlar body armour");

    DEF_SHIELD("small shield");
    DEF_SHIELD("large shield");
    DEF_SHIELD("riot shield");

    DEF_HEADGEAR("paper hat");
    DEF_HEADGEAR("paper crown");
    DEF_HEADGEAR("flat cap");
    DEF_HEADGEAR("beret");
    DEF_HEADGEAR("tam-o'-shanter");
    DEF_HEADGEAR("wooly hat");
    DEF_HEADGEAR("bobble hat");
    DEF_HEADGEAR("leather cap");
    DEF_HEADGEAR("peaked cap");
    DEF_HEADGEAR("skull cap");
    DEF_HEADGEAR("baseball cap");
    DEF_HEADGEAR("spiked helmet");
    DEF_HEADGEAR("crash helmet");
    DEF_HEADGEAR("hard hat");
    DEF_HEADGEAR("mortarboard");
    DEF_HEADGEAR("headscarf");
    DEF_HEADGEAR("wig");
    DEF_HEADGEAR("davy crocket hat");
    DEF_HEADGEAR("deerstalker");
    DEF_HEADGEAR("fez");
    DEF_HEADGEAR("top hat");
    DEF_HEADGEAR("trilby");
    DEF_HEADGEAR("bowler hat");
    DEF_HEADGEAR("steel helmet");
    DEF_HEADGEAR("crown");

    DEF_GLOVES("fingerless gloves");
    DEF_GLOVES("mittens");
    DEF_GLOVES("gloves");
    DEF_GLOVES("rigger's gloves");
    DEF_GLOVES("kevlar gloves");
    DEF_GLOVES("welder's gloves"); /* should that be welder's gauntlets?*/
    DEF_GLOVES("gauntlets");

    /* Good thing armour doesnt stack because neither do these names */
    DEF_FOOTWEAR("socks");
    DEF_FOOTWEAR("stockings");
    DEF_FOOTWEAR("slippers");
    DEF_FOOTWEAR("slipper-socks");
    DEF_FOOTWEAR("plimsolls");
    DEF_FOOTWEAR("sandals");
    DEF_FOOTWEAR("clogs");
    DEF_FOOTWEAR("trainers");
    DEF_FOOTWEAR("shoes");
    DEF_FOOTWEAR("low boots");
    DEF_FOOTWEAR("high boots");
    DEF_FOOTWEAR("cowboy boots");
    DEF_FOOTWEAR("thigh boots");
    DEF_FOOTWEAR("waders");
    DEF_FOOTWEAR("wellies");
    DEF_FOOTWEAR("roller skates");
    DEF_FOOTWEAR("roller blades");
    DEF_FOOTWEAR("ice skates");
    DEF_FOOTWEAR("steel toecapped boots");

    DEF_RING("ring");
    DEF_RING("magic ring"); /* what more do I do for these??? */

    DEF_NECKWEAR("necklace");
    DEF_NECKWEAR("amulet");
    DEF_NECKWEAR("torc");
    /* End of wieldables/wearables */

    /* Readables... Most of these are "flavoured" items */
    DEF_SCROLL("scroll"); /* er...*/

    DEF_BOOK("textbook");
    DEF_BOOK("novel"); /* just random stuff?*/
    DEF_BOOK("handbook"); /* gives some skills */
    DEF_BOOK("manual"); /* other skills*/
    DEF_BOOK("grimoire"); /* gives some spells */
    DEF_BOOK("codex"); /* other spells */
    /* End of readables */

    /* Comestibles!*/
    /* For most foods, thought must be given to how to number them */
    DEF_FOOD("sandwich");
    DEF_FOOD("scotch egg");
    DEF_FOOD("hard boiled egg");
    DEF_FOOD("pickle"); /* would this interfere with condiments?*/
    DEF_FOOD("fish"); /*should this be subdivided?*/
    DEF_FOOD("meat");
    DEF_FOOD("apple");
    DEF_FOOD("pancake");
    DEF_FOOD("orange");
    DEF_FOOD("doner kebab");
    DEF_FOOD("hamburger");
    DEF_FOOD("bread"); /* But *loaf* or *slice*? */
    DEF_FOOD("toast"); /* Must be slice, yes? */
    DEF_FOOD("banana");
    DEF_FOOD("shish kebab"); /* perhaps this could leave a skewer! */
    DEF_FOOD("noodles");
    DEF_FOOD("sushi");
    DEF_FOOD("meat pie");
    DEF_FOOD("quiche");
    DEF_FOOD("pizza");
    DEF_FOOD("pineapple");
    DEF_FOOD("soup"); /* things like these should have containers */
    DEF_FOOD("curry");
    DEF_FOOD("balti");
    DEF_FOOD("ice cream");
    DEF_FOOD("spam");
    DEF_FOOD("corned beef");
    DEF_FOOD("spaghetti");
    DEF_FOOD("rice");
    DEF_FOOD("packet of crisps");
    DEF_FOOD("banana chips");
    DEF_FOOD("bombay mix");
    DEF_FOOD("meatball");
    DEF_FOOD("haggis");
    DEF_FOOD("durian");
    DEF_FOOD("ham"); /* Specific meats */
    DEF_FOOD("gammon");
    DEF_FOOD("bacon");
    DEF_FOOD("beef");
    DEF_FOOD("steak");
    DEF_FOOD("lamb");
    DEF_FOOD("leg of lamb");
    DEF_FOOD("mutton");
    DEF_FOOD("pork chop");
    DEF_FOOD("lamb chop");
    DEF_FOOD("chicken breast");
    DEF_FOOD("turkey breast");
    DEF_FOOD("chicken leg");
    DEF_FOOD("turkey leg");
    DEF_FOOD("chicken wing");
    DEF_FOOD("turkey wing");
    DEF_FOOD("horse meat");  /* takes all sorts */
    DEF_FOOD("dog meat");  /* hmm */
    DEF_FOOD("dog food");
    DEF_FOOD("cat food");

    /* Corpse types should be defined by monster definitions?
       Should corpses just be lumped in with food (which means normal food)?*/
    DEF_CONDIMENT("salt");
    DEF_CONDIMENT("black pepper");
    DEF_CONDIMENT("white pepper");
    DEF_CONDIMENT("szechuan pepper");
    DEF_CONDIMENT("cayenne pepper");
    DEF_CONDIMENT("malt vinegar");
    DEF_CONDIMENT("spirit vinegar");
    DEF_CONDIMENT("balsamic vinegar"); /* think that's all vinegars*/
    DEF_CONDIMENT("english mustard");
    DEF_CONDIMENT("french mustard");
    DEF_CONDIMENT("wholegrain mustard");
    DEF_CONDIMENT("dijon mustard"); /* Any other mustards I forgot?*/
    DEF_CONDIMENT("horseradish");
    DEF_CONDIMENT("japanese horseradish"); /* wasabi? */
    DEF_CONDIMENT("coriander");
    DEF_CONDIMENT("basil"); /* there are a few types of basil*/
    DEF_CONDIMENT("oregano");
    DEF_CONDIMENT("parsley");
    DEF_CONDIMENT("thyme");
    DEF_CONDIMENT("lemongrass");
    DEF_CONDIMENT("paprika");
    DEF_CONDIMENT("cinnamon");
    DEF_CONDIMENT("ginger");
    DEF_CONDIMENT("nutmeg");
    DEF_CONDIMENT("anise");
    DEF_CONDIMENT("cardamom");
    DEF_CONDIMENT("cloves"); /* do these really count as condiments? */
    DEF_CONDIMENT("garlic powder");
    DEF_CONDIMENT("powdered onion"); /* to distinguish from any food onion */
    DEF_CONDIMENT("grated cheddar");
    DEF_CONDIMENT("grated parmesan");
    DEF_CONDIMENT("sliced lemon peel");
    DEF_CONDIMENT("pesto"); /* yes I know theres lots of types */
    DEF_CONDIMENT("salsa");
    DEF_CONDIMENT("spaghetti sauce");
    DEF_CONDIMENT("tomato ketchup");
    DEF_CONDIMENT("brown sauce");
    DEF_CONDIMENT("HP sauce");
    DEF_CONDIMENT("soy sauce");
    DEF_CONDIMENT("worcestershire sauce");
    DEF_CONDIMENT("tabasco sauce");
    DEF_CONDIMENT("chilli sauce");
    DEF_CONDIMENT("lemon juice");
    DEF_CONDIMENT("lime juice");

    /* Some potions should start off known. The player would
    probably know only some. What he recognises would not be
    made clear to him. If the player starts off *not* recognising
    such a potion, that potion would take on a different name
    as given below. If the player DID recognise the potion from
    the beginning, those alternative names could be used by OTHER
    potions. You might only find out that a "clear potion" wasn't
    water when you find a "potion of water". */
    DEF_POTION("water"); /* may be "clear potion"*/
    DEF_POTION("orange juice"); /* may be "orange potion" */
    DEF_POTION("cola"); /* may be "black potion" */
    DEF_POTION("lemonade"); /* may be "fizzy potion" */
    DEF_POTION("booze"); /* may be "brown potion" */
    DEF_POTION("tea"); /* may be "brown potion"? */
    DEF_POTION("coffee"); /* may be "murky potion"? */

    DEF_POTION("purple potion"); /* Unknown types should be */
    DEF_POTION("green potion"); /* distinguished by a flag that */
    DEF_POTION("blue potion"); /* makes the word "potion" */
    DEF_POTION("yellow potion"); /* go either at the end, or at */
    DEF_POTION("red potion"); /* the beginning with the word "of" */
    DEF_POTION("turquoise potion");
    DEF_POTION("orange potion");
    DEF_POTION("brown potion");
    DEF_POTION("violet potion");
    DEF_POTION("pink potion");
    DEF_POTION("viscous potion");
    DEF_POTION("oily potion");
    DEF_POTION("fizzy potion");
    DEF_POTION("bubbling potion");
    DEF_POTION("fuming potion");
    DEF_POTION("cloudy potion");
    DEF_POTION("murky potion");
    DEF_POTION("clear potion");
    DEF_POTION("opaque potion");
    DEF_POTION("grey potion");
    DEF_POTION("black potion");
    DEF_POTION("sparkling potion");
    DEF_POTION("glowing potion");
    DEF_POTION("flashing potion");
    DEF_POTION("swirly potion");
    DEF_POTION("pulsing potion");

    /* Zappables... Again, flavoured items. */
    DEF_WAND("magic wand"); /* uh... */
    DEF_ROD("magic rod"); /* ... */
    /* end of zappables */

    /* The rest... */ 
    DEF_TOOL("key");
    DEF_TOOL("remote control"); /* does nothing! :) */
    DEF_TOOL("pen");
    DEF_TOOL("screwdriver"); /* repair stuff? untrap stuff? */
    DEF_TOOL("wire cutters"); /* untrapping stuff */
    DEF_TOOL("scissors"); /* shrug */
    DEF_TOOL("shovel");
    DEF_TOOL("bottle"); /* Can be filled to get a potion! Or condiment? */

    DEF_CONTAINER("bucket"); /* for rocks! */
    DEF_CONTAINER("bag");
    DEF_CONTAINER("paper bag");
    DEF_CONTAINER("carrier bag");
    DEF_CONTAINER("satchel");
    DEF_CONTAINER("rucksack");
    DEF_CONTAINER("sack");
    DEF_CONTAINER("quiver"); /* for missiles */
    DEF_CONTAINER("wallet"); /* for coinage */
    DEF_CONTAINER("purse");  /* "  " */
    DEF_CONTAINER("money box"); /* " " */
    DEF_CONTAINER("safe"); /* " " , heavy!*/
    DEF_CONTAINER("cardboard box");
    DEF_CONTAINER("chest"); /* heavy! */
    DEF_CONTAINER("deposit box"); /* limited capacity!*/
    DEF_CONTAINER("cruet"); /* For condiments!!! */
    DEF_CONTAINER("mini fridge"); /* For food */
    DEF_CONTAINER("ice box"); /* " " , heavy! */
    DEF_CONTAINER("fridge freezer"); /* " " , V HEAVY! Hey, what about power?*/
    DEF_CONTAINER("chest freezer"); /* " ", V HEAVY! Ah, fudge it. */

    DEF_STONE("rock");
    DEF_STONE("pebble");
    DEF_STONE("flint");
    DEF_STONE("ore");
    DEF_STONE("gem");
    DEF_STONE("glass");

    DEF_COINAGE("gold coin");
    DEF_COINAGE("silver coin");
    DEF_COINAGE("brass coin");
    DEF_COINAGE("iron coin");
    DEF_COINAGE("thrupenny bit"); /* ;) */

}
