/* ==== item_t.h ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* Where do these get decleared? */
/* Our externs get declared in item_p.h, just because. */


/* itemgroup_t is now in enums.h */

typedef struct {
   int ToHit;
   int ToDam;
   int ToEv; /* How it helps you to Evade blows */
   int ToBl; /* How it helps you to Block blows */
} ww_base_t; /* Doesn't include jewelery any more. */
/* Should base types actually either be weapon or armour instead,
   and only ego items get the alternative stats?  Maybe, maybe not */
/* Hmm, normally weapon types have their base quality expressed as
   dice rolls. Armour normally has AC stats as fixed, absolute numbers...
   Improvements to fighting ability from armour is only normally
   implemented as plusses to your rolls. So I think that yes,
   weapons and armour should be treated separately (at least the
   base types)*/

/* Stat boosts given by jewellery can include hit/dam/ev/bl as stats.
   (like int, pow, str,dex, etc). Other types include "stat flags"
   that are status flags such as "invisible", "high metabolism", etc
   and can set or unset those flags. So such jewellery would have
   a flag bitmask and changer values.
   It becomes clear that the bitmask selects where the bits come
   from: 0 in bitmask selects the bits from the character, 1 in
   bitmask selects the bits from the jewellery.
   Question arises: How do cumulative effects of stat flag
   modifications work? If taken literally, the final effect
   would depend on the order of evaluation. */
typedef struct {
   int Stat_id; /* What stat is modified */
   /* Possible stats: tohit, todam, toev, tobl,
   speed, int, dex, con, str, pow, and others. NOT Hp, Mp, exp, etc.
   Values of Stat_id should be made an enum. */
   int Stat_mod; /* How is it modified? Vector */
} stat_mod_t; /* For jewellery, stat_mod should be 0? Or maximum value? */

typedef struct {
   int Status_bm; /* A bitmask: Which bits are changed */
   int Status_ch; /* What the bits are changed to */
} status_mod_t;

typedef struct {
   int Opens; /* Can you open/close it? */
   int Wlim; /* Max weight you can get into these containers */
   itemclass_t AcceptsClass; /* Superclass of items you can fit in it*/
   itemgroup_t AcceptsSpec; /* Specific group of items you can (also) fit in*/
} container_t;

union item_typedata_t {
   container_t Container; /* Container type info */
   ww_base_t Base; /* base stats for a wieldable/wearable */
   void (*Effect)(void); /* For potions, scrolls, gems, condiments */
};

typedef struct { /* ego item base types */
/* int Stat_id;  What stats are improved */
/* int Stat_mod;  How do we modify it? */
   stat_mod_t Statmod; /* As above. */
/* int Statflag_bm;  Stat flag bitmask */
/* int Statflag_chl  What the changed flags change to */
   status_mod_t Statusmod; /* As above */
   int Is_artifact; /* 1 if this is actually an artifact (different)*/
} ego_t;

typedef struct {
   int ToHit;  /* Enchantment */
   int ToDam;
   int ToEv;
   int ToBl;
/* int something;   Other stat modifications would come from Ego type */
   ego_t *Ego; /* Pointer if any to ego item type */
} ww_extras_t;

union item_inst_data_t {
   struct item_t *Contents; /* of a container type. But this needs def of item_t! */
   int Charges; /* of a wand/rod */
   ww_extras_t Extras; /* extra item specific data. Includes ptrs to ego items*/
};

