/* ==== structures.h ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define MAPWID 140
#define MAPHGT 140

/* tile_t is now in enums.h */

typedef struct {
    char Int;
    char Dex;
    char Con;
    char Str;
    char Exprate;
} fixed_stats_t;

typedef struct {
    int Hp; /* Hit points */
    int MaxHp;
    int Mp; /* Mana */
    int MaxMp;
    int Exp; /* Total experience points */
    int Level;
} var_stats_t;

typedef struct {
    int Mapno; /* Should be a pointer to map, but we aint got none yet! */
    int X;
    int Y;
} locid_t;



struct itemtype_t{
    char Symbol;
    int Type;
    char *Name;
    union item_typedata_t Typedata;  /*Data specific to this type */
    struct itemtype_t *Next; /* for the current implementation, we've
       too many types to count and then stick in an array */
};
typedef struct itemtype_t itemtype_t;

struct item_t {
    itemtype_t *Itemtype; /* Exactly what type of object? */
    unsigned int Number; /* How many of it? */
    char Invletter; /* If you've carried this before, this stores
       what letter it used in your inventory. */
    struct item_t *Next;  /* Next item in a list (if any) */
    union item_inst_data_t Instdata; /* data specific to this instance */
};
typedef struct item_t item_t;


struct mobtype_t{
    char Symbol; /* What letter is it? */
    int Colour; /* Should be changed to whatever type curses uses */
    char *Name;
    char *Desc; /* A description of this mob type */
    itemtype_t *Corpsetype; /* Pointer to the type for this mob's corpse */
    fixed_stats_t Fixed_stats; /* Global stats for this type, eg INT */
    struct mobtype_t *Next; /* Next in global list */
};
typedef struct mobtype_t mobtype_t;

typedef struct {
/*    locid_t Pos; */
    int X,Y;
    mobtype_t *Mobtype; /* Exactly what type of monster */
    var_stats_t Var_stats; /* Stats that change, eg HP, Mana*/
    item_t *Inventlist; /* List of any items carried */
} monster_t;


typedef struct {
    tile_t Tiletype; /* Could be walls, floors, etc*/
    monster_t *Mptr; /* Any monster that's here*/
    item_t *Itemlist; /* A list of any items here */
    char Seen; /*This tile has been seen at some point. Seen acts as a bool*/
    itemtype_t *Remembered; /*Pointer to item-type last seen here */
    locid_t *Dest; /* Pointer to any destination structure */
} maploc_t;

