/* ==== maparray.c ==== */
/* Now contains just the "public" functions, rest are in mapgen.c */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
/* #include <ncurses.h> Should be removed when debugging syms removed*/
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "memutils.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h" /*We need this, but maybe just for the #defines*/
#include "mg_utils.h"
#include "ajoin.h"
#include "item_p.h"
#include "crash.h"
#include "dXrng.h"
#include "ui.h" /* For error messages, which really ought to
                   be in crash.h */

/*
foogrid *map;
foogrid *tmap;
itemtype_t markerobs[16]; Just for testing.
roomdata_t room[217];  Unfortunately now global X(
area_t *firstarea=NULL; Pointer to the first area of list
  See all those? They're now in globals.c */
extern itemtype_t markerobs[];
extern foogrid *tmap;
extern area_t *firstarea;

dXrng_roller *d12;
dXrng_roller *d20; /* We'll use one of the dice arrays soon */

void mkmap_p1();
int mkmap_p2();
void mkmap_p3();
void mkmap_p4(int totalrooms);

void make_map()
{
   /*
   int x,y,dx,dy,mr,mxr,myr,n,flag,s,done=0;
   int tries,totalrooms,target;
   */

   int n,totalrooms; /* used here. */

/*   tile_t thistile;  used in tests */
/*   maploc_t * elem; */
/*   maploc_t *mapstart; */
   /* We should have already called initialise_dXrng by now */
   if(NULL==d12) d12=new_dX(12);
   if(NULL==d20) d20=new_dX(20);
   for(n=0;n<10;n++)
      new_itemtype(markerobs+n,48+n,"A marker number",0);
   for(n=10;n<16;n++)
      new_itemtype(markerobs+n,55+n,"A marker letter",0);

   /* We just call the subparts in order.
      This was just split up so we could profile it all
      and find out where the astronomical slowdown appears
      (it was something like 90% of the whole time) */
   mkmap_p1();
   totalrooms=mkmap_p2();
   mkmap_p3();
   mkmap_p4(totalrooms);
}

void mkmap_p1()
{
   int x,y;
   maploc_t *mapstart;
   maploc_t * elem;


   /* Shouldn't we only do this once??? */
   map=create_foogrid(MAPHGT,MAPWID,sizeof(maploc_t));
   tmap=create_foogrid(MAPHGT,MAPWID,sizeof(tunnellist_t*)); /*Pointers to em*/
   /* Now we populate that grid */
   mapstart=map->Data;

   for(x=0,elem=mapstart+FOOGRID_IDX(map,0,0); x<(MAPWID-1); elem++,x++)
   {
      elem->Tiletype=PTL_PWALL; /* Can't be passed or mined */
      elem->Mptr=NULL;
      elem->Seen=0;
      elem->Itemlist=NULL;
      elem->Dest=NULL;
   }
   for(x=0,elem=mapstart+FOOGRID_IDX(map,(MAPHGT-1),0); x<(MAPWID-1); elem++,x++)
   {
      elem->Tiletype=PTL_PWALL;
      elem->Mptr=NULL;
      elem->Seen=0;
      elem->Itemlist=NULL;
      elem->Dest=NULL;
   }
   for(y=1;y<MAPHGT-2;y++)
   {
      for(x=0,elem=mapstart+FOOGRID_IDX(map,y,0);x<(MAPWID-1);elem++,x++)
      {
         if(x==0||x==(MAPWID-1)) elem->Tiletype=PTL_PWALL; else elem->Tiletype=PTL_ROCK;
         elem->Mptr=NULL;
         elem->Seen=0;
         elem->Itemlist=NULL;
         elem->Dest=NULL;
      }
   }
}

/* Does phase 2 of make_map() routine */
int mkmap_p2()
{
   maploc_t *mapstart;
   maploc_t * elem;
   int n,s,x,y,dx,dy,flag, totalrooms;
   int tx,ty;/* Targets to scan to */

   mapstart=map->Data;
   /* Phase 2: Pick room centres (with enough space) */
   (mapstart+FOOGRID_IDX(map,MAPHGT/2,MAPWID/2))->Tiletype=PTL_RMID;
   room[216].Xcentre=MAPWID/2;
   room[216].Ycentre=MAPHGT/2;
   room[216].Area=(area_t *)counting_malloc(sizeof(area_t));
     /* Make sure there's a room in the middle (start position) */
   room[216].Area->Firstroom=216;
   room[216].Area->Firsttunnel=NULL;
   room[216].Area->Size=1;
   room[216].Area->Nextarea=NULL; /*End of area list. */
   firstarea=room[216].Area;
   totalrooms=1;

   /* Now we make a load more! */
   for(n=0;n<216;n++)
   {
      /* Pick some random spot, then check the squares +-6 squares around it
         for other spots that have already been marked. If we find any
         in *that* area, we don't use the spot we have, but look for another*/
      room[n].Xcentre=-1;room[n].Ycentre=-1; /* If we can't alloc one */
      room[n].Area=NULL;
      for(s=14;s>7;s--)
      {
        /* was using 8+roll_dX(d20)++roll_dX(d8)*/
        /* changed 8 and d8 to 6 and d6, with shrink to 140X map.*/
        x=10+( (6*roll_dX(d20))+(roll_dX(d12)%6) );
        y=10+( (6*roll_dX(d20))+(roll_dX(d12)%6) );
        flag=0;
        tx=x+s; ty=y+s; /* Don't calculate them *each time*! */
        for(dy=y-s;dy<=ty;dy++)
        {
           if(flag==1) break; /* Don't scan the whole bloody thing!*/
           if(dy<0||dy>MAPHGT-1) continue;
           elem=mapstart+FOOGRID_IDX(map,dy,x-s);
             /* Could this go out of range? Would it matter? */
           for(dx=x-s;dx<=tx;dx++,elem++)
           {
              if(dx<0||dx>MAPWID-1) continue;
               /* this doesn't make a room here illegal*/
       /*     if( (mapstart+FOOGRID_IDX(map,dy,dx))->Tiletype!=PTL_ROCK) */
              if( (elem)->Tiletype!=PTL_ROCK)
              {
                 /* We can't build a room around here. */ flag=1;
                 break;
              }
           }
        }
        if (flag==0)
        {
           (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_RMID;
           room[n].Xcentre=x;room[n].Ycentre=y;
           room[n].Area=(area_t *)counting_malloc(sizeof(area_t));
           room[n].Area->Firstroom=n;
           room[n].Area->Firsttunnel=NULL;
           room[n].Area->Size=1;
     /*    room[n].Area->Nextarea=room[216].Area->Nextarea;
           room[216].Area->Nextarea=room[n].Area;*/
           room[n].Area->Nextarea=firstarea;
           firstarea=room[n].Area;
           /*Attach this area to head
             of area list. Well, OK, 216's area is the head of the list.
             So this goes *behind* the head. Meh.*/
           totalrooms++;
           break; /* Don't look for smaller spaces now. */
        }
      }
   }
   return(totalrooms);
}

void mkmap_p3()
{
   int n,s, x,y,  dx,dy, flag;
   int mr,mxr,myr;
   tile_t thistile;
   maploc_t *mapstart=map->Data;
   maploc_t * elem;


   /* Phase 3: */
   /* Room midpoints established. Find their dimensions!*/
   for(n=0;n<217;n++)
   {
      if (-1==room[n].Xcentre) continue;
/*      add_item(map,x,y,markerobs+1); */
      for(s=14;s>7;s--) /* s of 6 would mean no space between walls*/
      {
         flag=0;
         for(dy=-s;dy<=s;dy++)
         {
            y=room[n].Ycentre+dy;
            if(y<0||y>MAPHGT-1) continue;
            elem=(mapstart+FOOGRID_IDX(map,y,room[n].Xcentre-s));
            for(dx=-s;dx<=s;dx++,elem++)
            {
               x=room[n].Xcentre+dx;
               if(x<0||x>MAPWID-1||(dx==0&&dy==0)) continue;
           /*  if( (mapstart+FOOGRID_IDX(map,y,x))->Tiletype==PTL_RMID) */
               if( elem->Tiletype==PTL_RMID)
               {
                   flag=1; /*We've encountered the middle of another room*/
                   break;
               }
            }
            if(flag==1) break;
         }
         if (flag==0) break; /*We have a room up to size s that's clear*/
      }
      /* So we now know the maximum size, but we have to check for
         other excavations, to reduce the maximum x and y size */
      mr=(4*s)/7; /* max possible radius */
      /* if s-mr<4 mr=s-4 */
      mr=((s-mr<5)?s-5:mr);
      add_item(map,room[n].Xcentre,room[n].Ycentre,markerobs+s);
      add_item(map,room[n].Xcentre+1,room[n].Ycentre,markerobs+mr);
     /* add_room_items(n); Need size of room before we do that. */
      do {
        flag=0;
        myr=((roll_dX(d12)*(mr-2))/17)+5; /* We say 5 to leave extra wall space*/
        /* rather than dividing by 11, we divide by 17 to give
           an aspect ratio that looks more square- in fact, more
           horizontal than square!*/
        mxr=((roll_dX(d12)*(mr-2))/11)+5; /* We take 3 off if it's safe*/
        /*myr=(mrmax of 2/3 s, min of 2) */
        /* mxr=(max of 2/3 s, min of 2) */
        /* radius of 2 gives room size (internal) of 5,  wall size of 7*/
        /* Could be optimised too */
        for(dy=-myr;dy<=myr;dy++)
           for(dx=-mxr;dx<=mxr;dx++)
           {
              x=room[n].Xcentre+dx;y=room[n].Ycentre+dy;
              if(x<1||x>(MAPWID-2)||y<1||y>(MAPHGT-2)) {flag=1; break;}
              if(dx==0&&dy==0) continue; /*The middle of the room */
              thistile=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
              if( thistile!=PTL_ROCK&&thistile!=PTL_HWALL&&thistile!=PTL_VWALL&&thistile!=PTL_CORNER)
              /* The perimeter walls *used* to be the same type as the rock*/
              {
                  flag=1;
                  break;
              }
           }
      }while(flag==1); /* we now have suitable dimensions! */
      myr=myr-3;mxr=mxr-3; /* Remove the wall compensation to draw it. */
      /*Proceed to dig out the room */
      /* This could be optimised too */
      for(dy=-myr;dy<=myr;dy++)
         for(dx=-mxr;dx<=mxr;dx++)
         {
            x=room[n].Xcentre+dx;y=room[n].Ycentre+dy;
            if((mapstart+FOOGRID_IDX(map,y,x))->Tiletype!=PTL_RMID) (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_FLOOR;
         }
      /* Hollowed out?*/
      room[n].Width=3+(mxr*2); /* mxr is horizontal radius, not width,
         and doesn't count the centre, nor the perimeter walls. */
      room[n].Height=3+(myr*2); /*Likewise*/
      room[n].Xstart=room[n].Xcentre-(mxr+1);
      room[n].Ystart=room[n].Ycentre-(myr+1);
      /* build the perimeter walls */
      dy=(myr+1);
      /* This could be optimised a little */
      for(dx=-mxr;dx<=mxr;dx++)
      {
         x=room[n].Xcentre+dx;y=room[n].Ycentre-dy;
         (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_HWALL;
         y=room[n].Ycentre+dy;
         (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_HWALL;
      }
      dx=(mxr+1);

      x=room[n].Xcentre-dx;y=room[n].Ycentre-dy;
      (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_CORNER;

      x=room[n].Xcentre+dx;
      (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_CORNER;

      y=room[n].Ycentre+dy;
      (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_CORNER;

      x=room[n].Xcentre-dx;
      (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_CORNER;

      /* Perhaps this could be optimised a tad? */
      for(dy=-myr;dy<=myr;dy++)
      {
         x=room[n].Xcentre-dx;y=room[n].Ycentre+dy;
         (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_VWALL;
         x=room[n].Xcentre+dx;
         (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_VWALL;
      }

      add_room_items(n);
      add_room_mobs(n);
      room[n].Nextroom=-1;
      /* Now we'd do stuff about links to other rooms. */
   }
}

/* Room for optimisation but doesn't seem to matter at all */
void mkmap_p4(int totalrooms)
{
   int done=0, n,tries,target;
   /* Just while debugging stuff */
   move(1,0);addstr("     ");
   move(2,0);addstr("   ");

   /* Phase 4: connect rooms so *everything* is accessible */
   while(done==0)
   {
       n=pick_room(totalrooms);
       for(tries=0;tries<5;tries++)
       {
          target=pick_room(totalrooms);
          if (target!=n&&(abs(room[n].Xcentre-room[target].Xcentre)<35)&&(abs(room[n].Ycentre-room[target].Ycentre)<35)&&(room[n].Area!=room[target].Area))
              /*EUREKA! We have one. Somehow. Indicate this fact*/
              tries=6; /*Do we still need to break here?*/
       }
       if(7==tries) /* perhaps this should be 7 if above is 6?? */
       {
          /* This means we have randomly found 2 rooms that
             are suitable for joining.*/
          if(room[n].Xcentre==-1) show_messages("Source room doesnt exist!");
          if(room[target].Xcentre==-1) show_messages("Target room doesnt exist!");
          attempt_joining(n,target); /*we won't *necessarily* do so!*/
          /* if path blocked by something not yet accessible from n,
            try to connect to that instead!*/
          /* else path must be blocked by something already accessible
          from n, so we try to steamroller past any tunnels we find
          to see if we can it a room accessible from n. Then we try
          to see if the path between that room and target it blocked.*/
       } else {
          /*We obviously failed to find a suitable target. Take
            a different approach. */
          /* find the smallest area. Then pick a room from that
          area that can make a tunnel to somewhere it cant yet
          access. Each time it hits a room already connected,
          restart the attempt from that room.*/
          n=pick_room_from_area(choose_small_area());
          for(tries=0;tries<15;tries++)/* Should be enough?*/
          {
             target=pick_room(totalrooms);
             if (target!=n && room[n].Area!=room[target].Area)
             /* We don't care where it is, if it's too far we'll
               connect to somewhere else anyway*/
                tries=19;
          }
          if (15==tries) /*We couldn't find one?? Search harder.*/
          {
             for(tries=0;tries<217;tries++)/*tries=roomno*/
             {
                if(tries!=n &&room[tries].Xcentre!=-1&&room[n].Area!=room[tries].Area)
                {
                   target=tries;
                   break;
                }
             }
             if(room[tries].Xcentre==-1)
             { show_messages("Picked nonexistent room!");crash(NO_MEM);}
             if (217==tries) crash(INCONSISTENT); /* We couldnt find one! */
          }
          if(target==-1) {show_messages("make_map:target=-1!");crash(NO_MEM);}
          attempt_joining(n,target);
       }

       /* If we find the first area we can find has a size of
         totalrooms, then everything is connected, so done=1;
         NB: room 216 is GUARANTEED to exist (room 0 is not!!!)
         so might as well test room[216].Area->Size for this.*/
/*      if ((room[216].Area->Size)>2) done++;
*/
       /* For the moment, we show which cues showed
          that everything was connected.
          small_area->Size>1 turned out to terminate map construction
          prematurely about one time in 20 or so, so I removed it. */
       if(firstarea->Size==totalrooms){move(1,0);addstr("first");done=1;}
       /* The first area contains all the rooms. */
       if(firstarea->Nextarea==NULL) {move(2,0);addstr("one");done=1;}
       /* The first area is the only area. */
       /* If at some point, some trigger and others dont, that's
         something to find out about.
         But "first" and "one" seem equivalent. */
   }
   /* We should now have everything fully accessible! */
   free_areas();/*Tidy up!*/
}


