/* ==== monster.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "monster.h"
#include "ui.h"
#include "item_p.h" /* For the corpses */
#include "memutils.h"

void add_mob(foogrid *map,int x,int y,mobtype_t *mtype)
{
   monster_t *newmob;
   long offset=FOOGRID_IDX(map,y,x);
   maploc_t *mapstart=map->Data;
   newmob=(monster_t *)counting_malloc(sizeof(monster_t));
   newmob->Mobtype=mtype;
   newmob->Inventlist=NULL; /*Carrying nothing.*/
   newmob->X=x;newmob->Y=y; /*Monster knows where it is*/
   newmob->Var_stats.MaxHp=10; /* Uh... */
   newmob->Var_stats.Hp = newmob->Var_stats.MaxHp;
   (mapstart+offset)->Mptr=newmob; /*Map knows where it is*/
/* return(newmob); We should return it again later when we get
   monsters that can move... */
}

void new_mobtype(mobtype_t *space, char sym, char *name,char *desc)
{
    int l;
    char *cname;
    space->Symbol=sym;
    space->Name=name;
    space->Desc=desc;
    l=1+sprintf(strbuf,"%s corpse",name);/* Ah. */
    cname=(char *)counting_malloc(l*sizeof(char));
    sprintf(cname,"%s corpse",name);
    push_itype(IG_CORPSE,cname); /* does allocation anyways*/
    space->Corpsetype=first_its[IG_CORPSE];
    /* unless push_itype changed, this should work */

}

void push_mtype(char sym,char *name)
{
    mobtype_t *newmt;
    newmt=(mobtype_t *)counting_malloc(sizeof(mobtype_t));
    newmt->Next=first_mt; /* Should have an array of mobtypes
       one for each symbol, like push_itype does */
    first_mt=newmt;
    mt_nums++;
    new_mobtype(newmt,sym,name,"A monster with no description");
}

void move_mob(foogrid *map, int dx, int dy, monster_t *mob)
{
    maploc_t *mapstart=map->Data;
    int x,y;
    long offset;
    tile_t tile;
    x=mob->X;y=mob->Y;
    offset=FOOGRID_IDX(map,y+dy,x+dx);
    tile=(mapstart+FOOGRID_IDX(map,y+dy,x+dx))->Tiletype;
    /* Commented whilst debugging map!!!  */
    if(tile==PTL_ROCK||tile==PTL_VWALL||tile==PTL_HWALL||tile==PTL_PWALL||tile==PTL_DOOR||tile==PTL_CORNER)
    {
       sprintf(strbuf,"%s walks into the %s!",mob->Mobtype->Name,tile==PTL_DOOR?"door":"wall");
       show_messages(strbuf);
       return;
    } /**/
    if (tile==PTL_PWALL) return; /* TEMPORARY!  Whilst above is commented */
    (mapstart+FOOGRID_IDX(map,y,x))->Mptr=NULL;
    x+=dx;y+=dy;
    mob->X=x;mob->Y=y;
    (mapstart+FOOGRID_IDX(map,y,x))->Mptr=mob;
}

void m_open_door(foogrid *map,monster_t *mob)
{
    int x=mob->X,y=mob->Y;
    int done=0;
    maploc_t *mapstart=map->Data;
    if((mapstart+FOOGRID_IDX(map,y-1,x))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y-1,x))->Tiletype=PTL_ODOOR;
       done=1;
    }
    if((mapstart+FOOGRID_IDX(map,y+1,x))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y+1,x))->Tiletype=PTL_ODOOR;
       done++;
    }
    if((mapstart+FOOGRID_IDX(map,y,x-1))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y,x-1))->Tiletype=PTL_ODOOR;
       done++;
    }
    if((mapstart+FOOGRID_IDX(map,y,x+1))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y,x+1))->Tiletype=PTL_ODOOR;
       done++;
    }
    if (done!=0)
    {
      sprintf(strbuf,"The %s opens the door%s",mob->Mobtype->Name,done>1?"s":"");
      show_messages(strbuf);
    } else show_messages("There's no door to open!");
}

void m_get_item(monster_t *mob,item_t *item)
{
    /* Assumes item is individual: item->Next MUST be NULL*/
    if (item==NULL) return; /*Play it safe*/
    item->Next=mob->Inventlist;
    mob->Inventlist=item;
}
item_t *m_discard_item(monster_t *mob)
{
    /* Removes top item from monster's inventory. Returns it
      as an individual item.*/
    item_t *item=mob->Inventlist;
    if (item==NULL) return(NULL); /*Plausible*/
    mob->Inventlist=item->Next;
    item->Next=NULL;
    return(item);
}

void m_die(foogrid *map, monster_t *victim)
{
    maploc_t *mapstart=map->Data;
    int x=victim->X, y=victim->Y;
    add_item(map,x,y,victim->Mobtype->Corpsetype);
    (mapstart+FOOGRID_IDX(map,y,x))->Mptr=NULL; /* KABOOM! */
    /* We should free this, yes? But if in a
    list of all monsters, need to free it from there too */
    counting_free(victim);
}

