/* ==== player.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "item_p.h" /* Used by p_pickup, which may be moved */
#include "player.h"
#include "ui.h"
#include "crash.h"
#include "combat.h"

/* player_t player; The player character's data
 Now in globals.c */

void init_player(foogrid *map,int x,int y)
{
/* maploc_t *mapstart=map->Data; */
   player.Inventlist=NULL;
   player.Symbol='@'; /* That's you! */
   player.X=x;player.Y=y;
/* (mapstart+FOOGRID_IDX(map,y,x))= ... Ah. New player char isn't pointer */
}


/* Adapted from move_mob */
void p_move(foogrid *map, int dx, int dy)
{
    maploc_t *mapstart=map->Data;
    maploc_t *elem;
    int x,y;
    monster_t *mptr;
  /*long offset; I don't need this!! */
    tile_t tile;
    x=player.X;y=player.Y;
    elem=(mapstart+FOOGRID_IDX(map,y+dy,x+dx));
    mptr=elem->Mptr;
    if(NULL!=mptr)
    {
      p_hit_m(map, mptr);
      return;
    }
    tile=elem->Tiletype;
    if(tile==PTL_ROCK||tile==PTL_VWALL||tile==PTL_HWALL||tile==PTL_PWALL||tile==PTL_DOOR||tile==PTL_CORNER)
    {
       sprintf(strbuf,"You walk into the %s!",tile==PTL_DOOR?"door":"wall");
       show_messages(strbuf);
       return;
    }
/*  (mapstart+FOOGRID_IDX(map,y,x))->Mptr=NULL; Not now */
    x+=dx;y+=dy;
    player.X=x;player.Y=y;
/*  (mapstart+FOOGRID_IDX(map,y,x))->Mptr=mob; Not now */
}


void p_open_door(foogrid *map) /*Change to ask which door to open */
{
    int x=player.X,y=player.Y;
    int done=0;
    maploc_t *mapstart=map->Data;
    if((mapstart+FOOGRID_IDX(map,y-1,x))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y-1,x))->Tiletype=PTL_ODOOR;
       done=1;
    }
    if((mapstart+FOOGRID_IDX(map,y+1,x))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y+1,x))->Tiletype=PTL_ODOOR;
       done++;
    }
    if((mapstart+FOOGRID_IDX(map,y,x-1))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y,x-1))->Tiletype=PTL_ODOOR;
       done++;
    }
    if((mapstart+FOOGRID_IDX(map,y,x+1))->Tiletype==PTL_DOOR)
    {
       (mapstart+FOOGRID_IDX(map,y,x+1))->Tiletype=PTL_ODOOR;
       done++;
    }
    if (done!=0)
    {
      sprintf(strbuf,"You open the door%s",done>1?"s":"");
      show_messages(strbuf);
    } else show_messages("There's no door to open!");
}

/* Change to put items into player inventory, merge items,
   and *set inventory letters* if appropriate */
void p_get_item(item_t *item) /* Needs new name */
{
    /* Assumes item is individual: item->Next MUST be NULL*/
    char letter;
    if (NULL==item) return; /*Play it safe*/

    letter=first_unused_invletter();
    if(letter==' ') { /* Your inv is full!*/
        crash(INCONSISTENT); /* we shouldn't have been called */
    }
    item->Next=player.Inventlist;
    item->Invletter=letter;
    tag_invletter_usage(letter,0);
    player.Inventlist=item;
}

/* Change to ask what to drop */
/* item_t *p_discard_item()  Needs new name */
void p_discard_item(item_t *item)
{
    /* Removes given item from player's inventory. That's it. */
 /* item_t *item; */
    item_t *prev;
/*  item=prompt_for_inv_item(); Now caller does that. */
/*  if (NULL==item) return(NULL); Plausible- No, now stupid*/
    if (NULL==item) crash(OUT_OF_BOUNDS); /* So there. */
    /* Now need some function that takes *item* from
       list (removing it) and passes it back. But if we
       take item from the head of the list, the pointer
       to the list has changed!!! So we check the
       head of list first.*/
    if (player.Inventlist==item)
    { /* The item is the head of list. */
       player.Inventlist=item->Next;
    } else { /* Search for it. */
       for(prev=player.Inventlist;prev->Next!=item&&prev->Next!=NULL;prev=prev->Next);
       if(prev->Next==NULL) { crash(INCONSISTENT);}
       prev->Next=item->Next;/* Snip item out */

    }
    /*
    player.Inventlist=item->Next; */
    item->Next=NULL;
    tag_invletter_usage(item->Invletter,1);
/*    return(item); We don't do that now. */
}

void p_pickup(foogrid *map) /* A combination of p_get_item,prompt_for_heap_item,skim_item*/
{
    maploc_t *mapstart=map->Data;
    item_t *item;
    item_t *heap=(mapstart+FOOGRID_IDX(map,player.Y,player.X))->Itemlist;
    if(NULL==heap) return; /* Nothing there! */

    /* At this point, we ought to check that we've got space, no? */

    if(heap->Next==NULL)
    { /* Only one item. Get it w/out question */
       skim_item(map,player.X,player.Y,heap); /* Removes that item from floor*/
       p_get_item(heap); /* Gives it to you! */
       item=heap;
    } else { /* More than one item. */
       item=prompt_for_heap_item(heap);
       if(NULL==item) return; /* You decided not to */
       skim_item(map,player.X,player.Y,item); /* Removes that item from heap*/
       p_get_item(item);
    }
    sprintf(strbuf,"You pick up the %s",item->Itemtype->Name);
    show_messages(strbuf);
}
void p_drop(foogrid *map)
{
/*  maploc_t *mapstart=map->Data; */
    item_t *item;
/*  item_t *heap=(mapstart+FOOGRID_IDX(map,player.Y,player.X))->Itemlist; */
    item=prompt_for_inv_item();
    if(NULL==item) return;
    p_discard_item(item);
    /*p_discard_item() *used* to prompt for the
    item to drop, remove it, and pass it back. Now, it just removes
    a given item. MUCH more useful. */
    place_item(map,player.X,player.Y,item);
    sprintf(strbuf,"You drop the %s.",item->Itemtype->Name);
    show_messages(strbuf);
}

