/* ==== ui.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/*#include <stdlib.h>
*/

#include <string.h> /* Was included from ui.h, but strcpy etc
   are mostly just used here. */
#include <ncurses.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "memutils.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h" /*We need this, but maybe just for the #defines*/
#include "item_p.h"
#include "crash.h"
#include "dXrng.h"
#include "los.h"
#include "player.h"
#include "ui.h"


/* char strbuf[160]; Now in globals.c */


void draw_map(int cornerx,int cornery,int px,int py)
{
   /* We assume that curses is already initialised. */
   int x,y,sx,sy,tile;
   maploc_t *elem;
   maploc_t *mapstart=map->Data;
   char sym;
   calc_los(map,px,py);
   for (y=cornery,sy=1;sy<23;y++,sy++)
      for (x=cornerx,elem=mapstart+FOOGRID_IDX(map,y,x),sx=30;sx<80;elem++,sx++,x++)
      {
          move(sy,sx);
          if (in_los(px,py,x,y))
          {
           elem->Seen=1;
           elem->Remembered=NULL; /* Update it whenever square seen */
           if(NULL!=elem->Mptr) sym=elem->Mptr->Mobtype->Symbol;
           else if (px==x&&py==y) sym=player.Symbol;
           else if (NULL!=elem->Itemlist)
           {
              sym=elem->Itemlist->Itemtype->Symbol;
              elem->Remembered=elem->Itemlist->Itemtype;
           } else{
             tile=elem->Tiletype; /* show the floor */
             switch(tile)
             {
                 case PTL_PWALL:
                 case PTL_CORNER:
                   sym='#';break;
                   sym='*';break;
                 case PTL_ROCK:
                   sym='#';break;
                 case PTL_HWALL:
                   sym='#';break;
                   sym='-';break;
                 case PTL_VWALL:
                   sym='#';break;
                   sym='|';break;
                 case PTL_DOOR:
                   sym='+';break;
                 case PTL_ODOOR:
                   sym=':';break; /* Er, I suppose*/
                 case PTL_TUNNEL:
                   sym=':';break;
                 case PTL_FLOOR:
                 case PTL_RMID:
                   sym='.';break;
                 default:
                   sym=''; /* Foobar!!!*/
             }
           }
          } else
          { /* draw from memory */
            if(elem->Seen)
            {
               if(NULL!=elem->Remembered) sym=elem->Remembered->Symbol;
               else switch (elem->Tiletype) {
                       case PTL_PWALL:
                       case PTL_CORNER:
                          sym='#';break;
                          sym='*';break;
                       case PTL_ROCK:
                          sym='#';break;
                       case PTL_HWALL:
                          sym='#';break;
                          sym='-';break;
                       case PTL_VWALL:
                          sym='#';break;
                          sym='|';break;
                       case PTL_DOOR:
                          sym='+';break;
                       case PTL_ODOOR:
                          sym=':';break;
                       case PTL_TUNNEL:
                          sym=':';break;
                       case PTL_FLOOR:
                       case PTL_RMID:
                          sym='.';break;
                       default:
                          sym='';
               }
            } else sym=' '; /* blank! */
          }
          addch(sym);
      }
   refresh();
}

void show_messages(char *msg)
{
   /* Message area starts from line 18, and runs from column 0
     to column 29*/
   /*char testbuf[50];  Use more than we need.*/
   char linebuf[29];
   char *startptr=msg;
   char *safeptr; /* When we find a bad line, this will be the end.*/
   char *testptr;
   int msglen=strlen(msg);
   int line;
   /* This should be a loop*/
   for (line=18;line<22;line++)
   {
      safeptr=startptr+strlen(startptr)-1;
      for(testptr=strchr(startptr,' ');
        testptr!=NULL&&(testptr-startptr)<28;
        safeptr=testptr+1,testptr=strchr(safeptr,' '));
    /*if (testptr==NULL)*/ msglen=strlen(startptr); 
      if (msglen>29) msglen=(safeptr-startptr)-1;
/* This needs much working out.*/
      if(msglen>29) crash(IO_ERROR); /*Arrgh!*/
      move(line,0);
      strncpy(linebuf,startptr,msglen);
      *(linebuf+msglen)='\0'; /*Add a null.*/
      strncat(linebuf,"                               ",29-msglen);
      addstr(linebuf);
      refresh(); /* Should be done at the END of the routine.*/
      if(testptr==NULL) break;
      startptr=safeptr;
   }
   for (line++;line<22;line++)
   {
      move(line,0);
      addstr("                              ");
   }
   /*
   if(msglen<29)
   {
      strcpy(linebuf,startptr);
      strncat(linebuf,"                               ",29-msglen);
      move(18,0);
      addstr(linebuf);
   }
   */
}

void msg_window(char *msg)
{
   int x,y;
   int length=strlen(msg);
   int xe=43+(length/2);
   for(x=37-(length/2);x<xe;x++)
   {
      move(8,x);addch('-');
      move(12,x);addch('-');
   }
   for(y=9;y<12;y++)
   {
      move(y,37-(length/2));addch('|');
      move(y,xe);addch('|');
   }
   move(8,37-(length/2));addch('+');
   move(12,37-(length/2));addch('+');
   move(8,xe);addch('+');
   move(12,xe);addch('+');
   move(10,40-(length/2));addstr(msg);
   move(14,0);
   refresh();
}

item_t *prompt_for_inv_item()
{ /* Tricky one. Takes list, displays it, asks you to pick one.*/
  /* Problem: Lists of items in your inventory are handled
     differently to items on the floor or in containers.
     Solution: give 2 functions:
         prompt_for_inv_item()
     and prompt_for_heap_item(item_t *list). */
/* int line;   Don't we use this?? */
   char k;
   item_t *item;
   show_messages("Which item? ? to choose from list.");
   while(1){
     k=getch();
     if (k=='?') { /* Show the list first */
        /*
        for(line=1,item=player.Inventlist;item!=NULL;line++,item=item->Next)
        {
           if(line<22) {move(line,0);
             addch(item->Invletter); addch(':');
             addstr(item->Itemtype->Name);}
        }
        refresh(); */
        display_heap("Items available",player.Inventlist); /* Tidier now */
     } else { /* Got a choice! If valid, return it. */
        if ((k>='a'&&k<='z')||(k>='A'&&k<='Z'))
        {
           for (item=player.Inventlist;item!=NULL;item=item->Next)
              if(item->Invletter==k) return(item);
           show_messages("You don't have *that*!");
        }
        if (k==27) /* ESC */ return(NULL);
     }
   }
}

item_t *prompt_for_heap_item(item_t *list)
{
   int line,items,choice;
   char k=' ';
   item_t *item;
   show_messages("Choose an item");
   while(list!=NULL) /* Er... */
   {
     for(items=0,item=list; items<15&&item!=NULL; items++,item=item->Next);
     for(line=0,item=list; line<items &&item!=NULL; line++, item=item->Next)
     {
        move(line+1,2);addch('|');
        addch(line+'a'); addch(':');
        addch(item->Itemtype->Symbol);addch(' ');
        addstr(item->Itemtype->Name);addstr("      "); /* Should be better.*/
     }
     for(;line<15;line++) /* Blank the rest */
     { move(line+1,3);addstr("                        "); /* That'll do. */
     }
     while(1)
     {
       k=getch();
       if ('a'<=k && ('a'+items)>k) /* Was source of foobar... */
       {  /* Got one. Which was it? */
          choice=(k-'a');
          for(line=0,item=list; line<choice&&item!=NULL;line++,item=item->Next);
          if(NULL==item) crash(FUNCTION_ERROR); /* Shouldn't happen */
          return(item);
       }
       if(27==k) return (NULL); /* Don't choose anything */
       if(' '==k) break; /* Next page. */
     }
     /* User must have pressed <space> to choose next page. */
     list=item;
   }
   /* GCC complains about control reaching here. Should it be
      returning NULL here? Or not reaching this point?? I'll look
      at it later. I suspect we should return NULL anyway. */
   return (NULL);
}

void display_heap(char *title,item_t *list)
{
   int count=0,x=0;
   item_t *scan;
   WINDOW *win;
   for(scan=list;scan!=NULL;scan=scan->Next)
      count++;
   if(count==0)return;
   /*
   move(8,1);
   addch('0'+(count/10));
   addch('0'+(count%10));
   refresh(); */
   win=newwin(count+4,60,9-(count/2),10);
   wmove(win,1,30-(strlen(title)/2));waddstr(win,title);
   for(count=0,scan=list;scan!=NULL;scan=scan->Next,count++)
   {
     wmove(win,count+2,0);waddch(win,'|');
     waddch(win,scan->Invletter);waddch(win,':');
     waddch(win,scan->Itemtype->Symbol);waddch(win,' ');
     waddstr(win,scan->Itemtype->Name);
     wmove(win,count+2,59);waddch(win,'|');
   }
   wmove(win,0,0);waddch(win,'+');
   wmove(win,0,59);waddch(win,'+');
   wmove(win,count+3,0);waddch(win,'+');
   wmove(win,count+3,59);waddch(win,'+');
   wmove(win,1,0);waddch(win,'|');
   wmove(win,1,59);waddch(win,'|');
   wmove(win,count+2,0);waddch(win,'|');
   wmove(win,count+2,59);waddch(win,'|');
   for(x=1;x<59;x++)
   {  wmove(win,0,x);waddch(win,'-');
      wmove(win,count+3,x);waddch(win,'-');
   }
   wmove(win,count+2,30-(strlen("Press <space> to continue")/2));
   waddstr(win,"Press <space> to continue");
   touchwin(win);
   wrefresh(win);
   while(' '!=getch())
   {}
   werase(win);wrefresh(win); /* blank it!*/
   delwin(win);

   touchwin(stdscr); /* I have to do this to restore what was under the */
   refresh(); /* window I just removed. Is this really the *only* way?? */

}
